"""
  Bookmarker for Idle - A simple Bookmarking facility for IDLE.
  Version 1.0, 2003
  Written by Andy Bulka abulka@netspace.net.au
  http://www.atug.com/andypatterns
 
  License: Free Software.

  Usage:
  ------
  An IDLE extension to store bookmarks as line numbers in an output window.
  To add a bookmark, select "Add Bookmark" from the edit menu or hit F2.
  To navigate to a bookmark, simply r.click on the line number you want.
  
  Installation:
  -------------
   1. Place this file in your ...python22/Lib/site-packages directory
   
   2. Edit your ...Tools/Idle/config-win.txt or config-unix.txt
     and add the entry
     [Bookmarker_Idle]
     
   3. To chanage the hot key, edit the source file and replace <F2>.
"""
import os
from Tkinter import*
import tkMessageBox
import sys
import string
import time
try:
    from idlelib.OutputWindow import OutputWindow
except ImportError:
    from OutputWindow import OutputWindow

class Bookmarker_Idle:

    menudefs = [
        ('edit', [
            None,
            ('_Add Bookmark', '<<bm-andy-bookmark>>'),
         ])
    ]

    keydefs = {
    '<<bm-andy-bookmark>>':['<F2>'], 
     }

##    windows_keydefs = {
##        '<<bm-andy-bookmark>>': ['<F2>'],
##    }
##    unix_keydefs = {
##        '<<bm-andy-bookmark>>': ['<Control-x><Control-2>'],
##    }

    def __init__(self, editwin):
        self.editwin = editwin
        self.bookmarksIO = None

    def bm_andy_bookmark_event(self,event):
        if not self.editwin.io.filename:
            self.errorbox("Bookmarker", "Please save your file before bookmarking.")
            return
        filename = os.path.normpath(self.editwin.io.filename)
        line, column = string.split(self.editwin.text.index("insert"), '.')

        selectedtext = curline = self.editwin.text.get("insert linestart", "insert lineend")
        selectedtext = selectedtext.strip()
        
        if not selectedtext:
            selectedtext = time.asctime()

        if self.bookmarksIO is None:
            self.bookmarksIO = BookmarksOutputWindow(self.editwin.flist, currentfilebeingbookmarked=filename)

        msg = 'File "%s", line %s, %s'%(filename,line,selectedtext)
        print >>self.bookmarksIO, msg
       

    def errorbox(self, title, message):
        tkMessageBox.showerror(title, message, master = self.editwin.text)
        self.editwin.text.focus_set()

    
class BookmarksOutputWindow(OutputWindow):
    def __init__(self, *args, **keywords):

        if keywords.has_key('currentfilebeingbookmarked'):
            file = keywords['currentfilebeingbookmarked']
            self.currentfilebeingbookmarked = os.path.basename(file)
            
        if not self.currentfilebeingbookmarked:    
            self.errorbox("Bookmarker Error", "Cannot determine what file we are in.")
            return

        OutputWindow.__init__(self, *args, **keywords)

    def short_title(self):
        return 'Bookmarks for ' + self.currentfilebeingbookmarked

